@@ -20,6 +20,7 @@ use uv_static::EnvVars;
 use crate::candidate_selector::CandidateSelector;
 use crate::dependency_provider::UvDependencyProvider;
 use crate::fork_urls::ForkUrls;
+use crate::prerelease::AllowPrerelease;
 use crate::pubgrub::{PubGrubPackage, PubGrubPackageInner, PubGrubReportFormatter};
 use crate::python_requirement::PythonRequirement;
 use crate::resolution::ConflictingDistributionError;
@@ -52,9 +53,6 @@ pub enum ResolveError {
         extra: ExtraName,
     },
 
-    #[error("Overrides contain conflicting URLs for package `{0}`:\n- {1}\n- {2}")]
-    ConflictingOverrideUrls(PackageName, String, String),
-
     #[error(
         "Requirements contain conflicting URLs for package `{package_name}`{}:\n- {}",
         if env.marker_environment().is_some() {
@@ -338,6 +336,20 @@ impl std::fmt::Display for NoSolutionError {
         collapse_unavailable_versions(&mut tree);
         collapse_redundant_depends_on_no_versions(&mut tree);
 
+        simplify_derivation_tree_ranges(
+            &mut tree,
+            &self.available_versions,
+            &self.selector,
+            &self.env,
+        );
+
+        // This needs to be applied _after_ simplification of the ranges
+        collapse_redundant_no_versions(&mut tree);
+
+        while collapse_redundant_no_versions_tree(&mut tree) {
+            // Continue collapsing until no more redundant nodes are found
+        }
+
         if should_display_tree {
             display_tree(&tree, "Resolver derivation tree after reduction");
         }
@@ -358,7 +370,7 @@ impl std::fmt::Display for NoSolutionError {
             &self.fork_urls,
             &self.env,
             &self.workspace_members,
-            self.options,
+            &self.options,
             &mut additional_hints,
         );
         for hint in additional_hints {
@@ -390,32 +402,175 @@ fn display_tree_inner(
     lines: &mut Vec<String>,
     depth: usize,
 ) {
+    let prefix = "  ".repeat(depth).to_string();
     match error {
         DerivationTree::Derived(derived) => {
             display_tree_inner(&derived.cause1, lines, depth + 1);
             display_tree_inner(&derived.cause2, lines, depth + 1);
+            for (package, term) in &derived.terms {
+                match term {
+                    Term::Positive(versions) => {
+                        lines.push(format!("{prefix}term {package}{versions}"));
+                    }
+                    Term::Negative(versions) => {
+                        lines.push(format!("{prefix}term not {package}{versions}"));
+                    }
+                }
+            }
         }
-        DerivationTree::External(external) => {
-            let prefix = "  ".repeat(depth).to_string();
-            match external {
-                External::FromDependencyOf(package, version, dependency, dependency_version) => {
-                    lines.push(format!(
-                        "{prefix}{package}{version} depends on {dependency}{dependency_version}"
-                    ));
+        DerivationTree::External(external) => match external {
+            External::FromDependencyOf(package, version, dependency, dependency_version) => {
+                lines.push(format!(
+                    "{prefix}{package}{version} depends on {dependency}{dependency_version}"
+                ));
+            }
+            External::Custom(package, versions, reason) => match reason {
+                UnavailableReason::Package(_) => {
+                    lines.push(format!("{prefix}{package} {reason}"));
                 }
-                External::Custom(package, versions, reason) => match reason {
-                    UnavailableReason::Package(_) => {
-                        lines.push(format!("{prefix}{package} {reason}"));
-                    }
-                    UnavailableReason::Version(_) => {
-                        lines.push(format!("{prefix}{package}{versions} {reason}"));
-                    }
-                },
-                External::NoVersions(package, versions) => {
-                    lines.push(format!("{prefix}no versions of {package}{versions}"));
+                UnavailableReason::Version(_) => {
+                    lines.push(format!("{prefix}{package}{versions} {reason}"));
                 }
-                External::NotRoot(package, versions) => {
-                    lines.push(format!("{prefix}not root {package}{versions}"));
+            },
+            External::NoVersions(package, versions) => {
+                lines.push(format!("{prefix}no versions of {package}{versions}"));
+            }
+            External::NotRoot(package, versions) => {
+                lines.push(format!("{prefix}not root {package}{versions}"));
+            }
+        },
+    }
+}
+
+fn collapse_redundant_no_versions(
+    tree: &mut DerivationTree<PubGrubPackage, Range<Version>, UnavailableReason>,
+) {
+    match tree {
+        DerivationTree::External(_) => {}
+        DerivationTree::Derived(derived) => {
+            match (
+                Arc::make_mut(&mut derived.cause1),
+                Arc::make_mut(&mut derived.cause2),
+            ) {
+                // If we have a node for a package with no versions...
+                (
+                    DerivationTree::External(External::NoVersions(package, versions)),
+                    ref mut other,
+                )
+                | (
+                    ref mut other,
+                    DerivationTree::External(External::NoVersions(package, versions)),
+                ) => {
+                    // First, always recursively visit the other side of the tree
+                    collapse_redundant_no_versions(other);
+
+                    // Retrieve the nearest terms, either alongside this node or from the parent.
+                    let package_terms = if let DerivationTree::Derived(derived) = other {
+                        derived.terms.get(package)
+                    } else {
+                        derived.terms.get(package)
+                    };
+
+                    let Some(Term::Positive(term)) = package_terms else {
+                        return;
+                    };
+
+                    let versions = versions.complement();
+
+                    // If we're disqualifying a single version, this is important to retain, e.g,
+                    // for `only foo==1.0.0 is available`
+                    if versions.as_singleton().is_some() {
+                        return;
+                    }
+
+                    // If the range in the conclusion (terms) matches the range of no versions,
+                    // then we'll drop this node. If the range is "all versions", then there's no
+                    // also no need to enumerate the available versions.
+                    if *term != Range::full() && *term != versions {
+                        return;
+                    }
+
+                    *tree = other.clone();
+                }
+                // If not, just recurse
+                _ => {
+                    collapse_redundant_no_versions(Arc::make_mut(&mut derived.cause1));
+                    collapse_redundant_no_versions(Arc::make_mut(&mut derived.cause2));
+                }
+            }
+        }
+    }
+}
+
+/// Given a [`DerivationTree`], collapse any derived trees with two `NoVersions` nodes for the same
+/// package. For example, if we have a tree like:
+///
+/// ```text
+/// term Python>=3.7.9
+///   no versions of Python>=3.7.9, <3.8
+///   no versions of Python>=3.8
+/// ```
+///
+/// We can simplify this to:
+///
+/// ```text
+/// no versions of Python>=3.7.9
+/// ```
+///
+/// This function returns a `bool` indicating if a change was made. This allows for repeated calls,
+/// e.g., the following tree contains nested redundant trees:
+///
+/// ```text
+/// term Python>=3.10
+///   no versions of Python>=3.11, <3.12
+///   term Python>=3.10, <3.11 | >=3.12
+///     no versions of Python>=3.12
+///     no versions of Python>=3.10, <3.11
+/// ```
+///
+/// We can simplify this to:
+///
+/// ```text
+/// no versions of Python>=3.10
+/// ```
+///
+/// This appears to be common with the way the resolver currently models Python version
+/// incompatibilities.
+fn collapse_redundant_no_versions_tree(
+    tree: &mut DerivationTree<PubGrubPackage, Range<Version>, UnavailableReason>,
+) -> bool {
+    match tree {
+        DerivationTree::External(_) => false,
+        DerivationTree::Derived(derived) => {
+            match (
+                Arc::make_mut(&mut derived.cause1),
+                Arc::make_mut(&mut derived.cause2),
+            ) {
+                // If we have a tree with two `NoVersions` nodes for the same package...
+                (
+                    DerivationTree::External(External::NoVersions(package, versions)),
+                    DerivationTree::External(External::NoVersions(other_package, other_versions)),
+                ) if package == other_package => {
+                    // Retrieve the terms from the parent.
+                    let Some(Term::Positive(term)) = derived.terms.get(package) else {
+                        return false;
+                    };
+
+                    // If they're both subsets of the term, then drop this node in favor of the term
+                    if versions.subset_of(term) && other_versions.subset_of(term) {
+                        *tree = DerivationTree::External(External::NoVersions(
+                            package.clone(),
+                            term.clone(),
+                        ));
+                        return true;
+                    }
+
+                    false
+                }
+                // If not, just recurse
+                _ => {
+                    collapse_redundant_no_versions_tree(Arc::make_mut(&mut derived.cause1))
+                        || collapse_redundant_no_versions_tree(Arc::make_mut(&mut derived.cause2))
                 }
             }
         }
@@ -939,3 +1094,168 @@ impl std::fmt::Display for NoSolutionHeader {
         }
     }
 }
+
+/// Given a [`DerivationTree`], simplify version ranges using the available versions for each
+/// package.
+fn simplify_derivation_tree_ranges(
+    tree: &mut DerivationTree<PubGrubPackage, Range<Version>, UnavailableReason>,
+    available_versions: &FxHashMap<PackageName, BTreeSet<Version>>,
+    candidate_selector: &CandidateSelector,
+    resolver_environment: &ResolverEnvironment,
+) {
+    match tree {
+        DerivationTree::External(external) => match external {
+            External::FromDependencyOf(package1, versions1, package2, versions2) => {
+                if let Some(simplified) = simplify_range(
+                    versions1,
+                    package1,
+                    available_versions,
+                    candidate_selector,
+                    resolver_environment,
+                ) {
+                    *versions1 = simplified;
+                }
+                if let Some(simplified) = simplify_range(
+                    versions2,
+                    package2,
+                    available_versions,
+                    candidate_selector,
+                    resolver_environment,
+                ) {
+                    *versions2 = simplified;
+                }
+            }
+            External::NoVersions(package, versions) => {
+                if let Some(simplified) = simplify_range(
+                    versions,
+                    package,
+                    available_versions,
+                    candidate_selector,
+                    resolver_environment,
+                ) {
+                    *versions = simplified;
+                }
+            }
+            External::Custom(package, versions, _) => {
+                if let Some(simplified) = simplify_range(
+                    versions,
+                    package,
+                    available_versions,
+                    candidate_selector,
+                    resolver_environment,
+                ) {
+                    *versions = simplified;
+                }
+            }
+            External::NotRoot(..) => (),
+        },
+        DerivationTree::Derived(derived) => {
+            // Recursively simplify both sides of the tree
+            simplify_derivation_tree_ranges(
+                Arc::make_mut(&mut derived.cause1),
+                available_versions,
+                candidate_selector,
+                resolver_environment,
+            );
+            simplify_derivation_tree_ranges(
+                Arc::make_mut(&mut derived.cause2),
+                available_versions,
+                candidate_selector,
+                resolver_environment,
+            );
+
+            // Simplify the terms
+            derived.terms = std::mem::take(&mut derived.terms)
+                .into_iter()
+                .map(|(pkg, term)| {
+                    let term = match term {
+                        Term::Positive(versions) => Term::Positive(
+                            simplify_range(
+                                &versions,
+                                &pkg,
+                                available_versions,
+                                candidate_selector,
+                                resolver_environment,
+                            )
+                            .unwrap_or(versions),
+                        ),
+                        Term::Negative(versions) => Term::Negative(
+                            simplify_range(
+                                &versions,
+                                &pkg,
+                                available_versions,
+                                candidate_selector,
+                                resolver_environment,
+                            )
+                            .unwrap_or(versions),
+                        ),
+                    };
+                    (pkg, term)
+                })
+                .collect();
+        }
+    }
+}
+
+/// Helper function to simplify a version range using available versions for a package.
+///
+/// If the range cannot be simplified, `None` is returned.
+fn simplify_range(
+    range: &Range<Version>,
+    package: &PubGrubPackage,
+    available_versions: &FxHashMap<PackageName, BTreeSet<Version>>,
+    candidate_selector: &CandidateSelector,
+    resolver_environment: &ResolverEnvironment,
+) -> Option<Range<Version>> {
+    // If there's not a package name or available versions, we can't simplify anything
+    let name = package.name()?;
+    let versions = available_versions.get(name)?;
+
+    // If this is a full range, there's nothing to simplify
+    if range == &Range::full() {
+        return None;
+    }
+
+    // If there's only one version available and it's in the range, return just that version
+    if let Some(version) = versions.iter().next() {
+        if versions.len() == 1 && range.contains(version) {
+            return Some(Range::singleton(version.clone()));
+        }
+    }
+
+    // Check if pre-releases are allowed
+    let prereleases_not_allowed = candidate_selector
+        .prerelease_strategy()
+        .allows(name, resolver_environment)
+        != AllowPrerelease::Yes;
+
+    let any_prerelease = range.iter().any(|(start, end)| {
+        let is_pre1 = match start {
+            Bound::Included(version) => version.any_prerelease(),
+            Bound::Excluded(version) => version.any_prerelease(),
+            Bound::Unbounded => false,
+        };
+        let is_pre2 = match end {
+            Bound::Included(version) => version.any_prerelease(),
+            Bound::Excluded(version) => version.any_prerelease(),
+            Bound::Unbounded => false,
+        };
+        is_pre1 || is_pre2
+    });
+
+    // Simplify the range, as implemented in PubGrub
+    Some(range.simplify(versions.iter().filter(|version| {
+        // If there are pre-releases in the range segments, we need to include pre-releases
+        if any_prerelease {
+            return true;
+        }
+
+        // If pre-releases are not allowed, filter out pre-releases
+        if prereleases_not_allowed && version.any_prerelease() {
+            return false;
+        }
+
+        // Otherwise, include the version
+        true
+    })))
+}
