@@ -497,7 +497,7 @@ pub struct ResolverInstallerOptions {
     /// preferring older versions that are compatible with a wider range of supported Python
     /// versions or platforms.
     #[option(
-        default = "\"fewest\"",
+        default = "\"requires-python\"",
         value_type = "str",
         example = r#"
             fork-strategy = "fewest"
@@ -709,16 +709,16 @@ pub struct ResolverInstallerOptions {
 pub struct PythonInstallMirrors {
     /// Mirror URL for downloading managed Python installations.
     ///
-    /// By default, managed Python installations are downloaded from [`python-build-standalone`](https://github.com/indygreg/python-build-standalone).
+    /// By default, managed Python installations are downloaded from [`python-build-standalone`](https://github.com/astral-sh/python-build-standalone).
     /// This variable can be set to a mirror URL to use a different source for Python installations.
-    /// The provided URL will replace `https://github.com/indygreg/python-build-standalone/releases/download` in, e.g., `https://github.com/indygreg/python-build-standalone/releases/download/20240713/cpython-3.12.4%2B20240713-aarch64-apple-darwin-install_only.tar.gz`.
+    /// The provided URL will replace `https://github.com/astral-sh/python-build-standalone/releases/download` in, e.g., `https://github.com/astral-sh/python-build-standalone/releases/download/20240713/cpython-3.12.4%2B20240713-aarch64-apple-darwin-install_only.tar.gz`.
     ///
     /// Distributions can be read from a local directory by using the `file://` URL scheme.
     #[option(
         default = "None",
         value_type = "str",
         example = r#"
-            python-install-mirror = "https://github.com/indygreg/python-build-standalone/releases/download"
+            python-install-mirror = "https://github.com/astral-sh/python-build-standalone/releases/download"
         "#
     )]
     pub python_install_mirror: Option<String>,
@@ -1099,7 +1099,7 @@ pub struct PipOptions {
     /// preferring older versions that are compatible with a wider range of supported Python
     /// versions or platforms.
     #[option(
-        default = "\"fewest\"",
+        default = "\"requires-python\"",
         value_type = "str",
         example = r#"
             fork-strategy = "fewest"
