@@ -15,8 +15,8 @@ use uv_cache_key::RepositoryUrl;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, Constraints, DevGroupsManifest, DevGroupsSpecification, DevMode, EditableMode,
-    ExtrasSpecification, GroupsSpecification, InstallOptions, LowerBound, SourceStrategy,
-    TrustedHost,
+    ExtrasSpecification, GroupsSpecification, InstallOptions, LowerBound, PreviewMode,
+    SourceStrategy, TrustedHost,
 };
 use uv_dispatch::{BuildDispatch, SharedState};
 use uv_distribution::DistributionDatabase;
@@ -77,6 +77,7 @@ pub(crate) async fn add(
     script: Option<PathBuf>,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
+    installer_metadata: bool,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
@@ -84,6 +85,7 @@ pub(crate) async fn add(
     no_config: bool,
     cache: &Cache,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<ExitStatus> {
     for source in &requirements {
         match source {
@@ -339,6 +341,7 @@ pub(crate) async fn add(
         bounds,
         sources,
         concurrency,
+        preview,
     );
 
     // Resolve any unnamed requirements.
@@ -673,12 +676,14 @@ pub(crate) async fn add(
         raw_sources,
         settings.as_ref(),
         bounds,
+        installer_metadata,
         connectivity,
         concurrency,
         native_tls,
         allow_insecure_host,
         cache,
         printer,
+        preview,
     )
     .await
     {
@@ -715,12 +720,14 @@ async fn lock_and_sync(
     raw_sources: bool,
     settings: ResolverInstallerSettingsRef<'_>,
     bounds: LowerBound,
+    installer_metadata: bool,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
     allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<(), ProjectError> {
     let mode = if locked {
         LockMode::Locked(environment.interpreter())
@@ -741,6 +748,7 @@ async fn lock_and_sync(
         allow_insecure_host,
         cache,
         printer,
+        preview,
     )
     .await?
     .into_lock();
@@ -859,6 +867,7 @@ async fn lock_and_sync(
                 allow_insecure_host,
                 cache,
                 printer,
+                preview,
             )
             .await?
             .into_lock();
@@ -918,12 +927,14 @@ async fn lock_and_sync(
         Modifications::Sufficient,
         settings.into(),
         Box::new(DefaultInstallLogger),
+        installer_metadata,
         connectivity,
         concurrency,
         native_tls,
         allow_insecure_host,
         cache,
         printer,
+        preview,
     )
     .await?;
 
