@@ -10,7 +10,7 @@ use uv_cache::Cache;
 use uv_client::{Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, Constraints, DevGroupsManifest, DevGroupsSpecification, EditableMode,
-    ExtrasSpecification, HashCheckingMode, InstallOptions, LowerBound, TrustedHost,
+    ExtrasSpecification, HashCheckingMode, InstallOptions, LowerBound, PreviewMode, TrustedHost,
 };
 use uv_dispatch::{BuildDispatch, SharedState};
 use uv_distribution_types::{
@@ -59,6 +59,7 @@ pub(crate) async fn sync(
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
     settings: ResolverInstallerSettings,
+    installer_metadata: bool,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
@@ -66,6 +67,7 @@ pub(crate) async fn sync(
     no_config: bool,
     cache: &Cache,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<ExitStatus> {
     // Identify the project.
     let project = if frozen {
@@ -155,6 +157,7 @@ pub(crate) async fn sync(
         allow_insecure_host,
         cache,
         printer,
+        preview,
     )
     .await
     {
@@ -223,12 +226,14 @@ pub(crate) async fn sync(
         modifications,
         settings.as_ref().into(),
         Box::new(DefaultInstallLogger),
+        installer_metadata,
         connectivity,
         concurrency,
         native_tls,
         allow_insecure_host,
         cache,
         printer,
+        preview,
     )
     .await
     {
@@ -256,12 +261,14 @@ pub(super) async fn do_sync(
     modifications: Modifications,
     settings: InstallerSettingsRef<'_>,
     logger: Box<dyn InstallLogger>,
+    installer_metadata: bool,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
     allow_insecure_host: &[TrustedHost],
     cache: &Cache,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<(), ProjectError> {
     // Use isolated state for universal resolution. When resolving, we don't enforce that the
     // prioritized distributions match the current platform. So if we lock here, then try to
@@ -422,6 +429,7 @@ pub(super) async fn do_sync(
         bounds,
         sources,
         concurrency,
+        preview,
     );
 
     let site_packages = SitePackages::from_environment(venv)?;
@@ -446,6 +454,7 @@ pub(super) async fn do_sync(
         cache,
         venv,
         logger,
+        installer_metadata,
         dry_run,
         printer,
     )
