@@ -276,7 +276,7 @@ fn tool_list_show_version_specifiers() {
 
     uv_snapshot!(context.filters(), context.tool_list().arg("--show-version-specifiers")
     .env(EnvVars::UV_TOOL_DIR, tool_dir.as_os_str())
-    .env(EnvVars::XDG_BIN_HOME, bin_dir.as_os_str()), @r#"
+    .env(EnvVars::XDG_BIN_HOME, bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -287,12 +287,12 @@ fn tool_list_show_version_specifiers() {
     - flask
 
     ----- stderr -----
-    "#);
+    "###);
 
     // with paths
     uv_snapshot!(context.filters(), context.tool_list().arg("--show-version-specifiers").arg("--show-paths")
     .env(EnvVars::UV_TOOL_DIR, tool_dir.as_os_str())
-    .env(EnvVars::XDG_BIN_HOME, bin_dir.as_os_str()), @r#"
+    .env(EnvVars::XDG_BIN_HOME, bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -303,5 +303,5 @@ fn tool_list_show_version_specifiers() {
     - flask ([TEMP_DIR]/bin/flask)
 
     ----- stderr -----
-    "#);
+    "###);
 }
